#include <Arduino.h>

const int ZERO_PIN = 0;
const int ONE_PIN = 1;
char data[35];
boolean valid = false;
unsigned long timestamp = 0;
int curr_bit = 0;

void setup();
void loop();
void zero_bit();
void one_bit();
int parse();
bool fullParity();
bool evenParity();
bool oddParity();

void setup() {
  attachInterrupt(ZERO_PIN,zero_bit,FALLING);
  attachInterrupt(ONE_PIN,one_bit,FALLING);
  Serial.begin(9600);
}

void loop() {
  if(valid == true) {
    noInterrupts();
    for(int i = 0; i < 35; i++) {
      Serial.print(data[i]);
    }
    Serial.println();
    curr_bit = 0;
    valid = false;

    int cardnum = parse();
    if(cardnum != -1) {
      Serial.print("Card Number: ");
      Serial.println(cardnum);
    } else {
      Serial.println("Invalid Card Number");
    }

    interrupts();
  }
}

void zero_bit() {
  if((valid == true) || (millis() - timestamp > 100)) {
    valid = false;
    curr_bit = 0;
  }
  data[curr_bit++] = '0';
  if(curr_bit>34) {
    valid = true;
  }
  timestamp = millis();
}

void one_bit() {
if((valid == true) || (millis() - timestamp > 100)) {
    valid = false;
    curr_bit = 0;
  }
  data[curr_bit++] = '1';
  if(curr_bit>34) {
    valid = true;
  }
  timestamp = millis();
}

int parse() {
  if (sizeof(data) == 35) {
    if (fullParity() && evenParity() && oddParity()) {
      char cardnum[20];
      for (int i = 14; i < 34; i++) cardnum[i-14] = data[i];
      return strtol(cardnum, NULL, 2);
    }
  }

  return -1;
}

bool fullParity() {
  int num = 0;
  for (int i = 0; i < 34; i++) {
    if (data[i] == '1') num++;
  }

  if (num % 2 == 0) {
    Serial.println("Full parity check passed");
    return true;
  }
  Serial.println("Full parity check failed");
  return false;  
}

bool evenParity() {
  int num = 0;
  
  for (int i = 2; i < 34; i++) {
    if (i == 4 || i == 10 || i == 16 || i == 22 || i == 28) continue;
    if (data[i] == '1') num++;
  }

  if (num % 2 == 0) {
    Serial.println("Even parity check passed");
    return true;
  }
  Serial.println("Even parity check failed");
  return false;  
}

bool oddParity() {
  int num = 0;
  
  for (int i = 2; i < 34; i++) {
    if (i == 5 || i == 9 || i == 13 || i == 17 || i == 21 || i == 25 || i == 29 || i == 33) continue;
    if (data[i] == '1') num++;
  }

  if (num % 2 == 0) {
    Serial.println("Odd parity check passed");
    return true;
  }
  Serial.println("Odd parity check failed");
  return false;   
}